#python imports
from typing import List
from fastapi import APIRouter, HTTPException
from fastapi.responses import JSONResponse
from fastapi import FastAPI, Request
from fastapi.responses import HTMLResponse
from pydantic import BaseModel

#dependency imports
from entities.product import Product
from use_cases.view_all_products import ViewAllProducts, IViewAllProductsInputPort, IViewAllProductsOutputPort
from use_cases.view_product import ViewProduct, IViewProductInputPort, IViewProductOutputPort
from interface_adapters.sql_product_repository import SQLProductRepository

class ProductDTO(BaseModel):
    id: int
    name: str
    description: str
    price: float

class ViewProductRequest(BaseModel):
    product_id: int

class ViewProductResponse(BaseModel):
    status: str
    data: ProductDTO

class ViewProductPresenter(IViewProductOutputPort):
    def present(self, product:Product) -> ViewProductResponse:
        """Format the response data for API."""
        product_dto = ProductDTO(
            id = product.id, 
            name = product.name, 
            description = product.description, 
            price = product.price)
        response = ViewProductResponse(status="success",data=product_dto)
        return response

class ViewProductController:
    def __init__(self, input_port: IViewProductInputPort) -> None:
        self.input_port = input_port

    def handle_request(self, request:ViewProductRequest):
        """Handles the incoming API request and delegates to the use case."""
        return self.input_port.execute(request.product_id)
    
product_repository = SQLProductRepository()
view_product_presenter = ViewProductPresenter()
view_product_use_case = ViewProduct(product_repository, view_product_presenter)
view_product_controller = ViewProductController(view_product_use_case)


class ViewAllProductsResponse(BaseModel):
    status: str
    data: List[ProductDTO]

class ViewAllProductsPresenter(IViewAllProductsOutputPort):
    def present(self, products:List[Product]) -> ViewAllProductsResponse:
        """Format the response data for API."""
        products_dto=[ProductDTO(id=product.id, name=product.name, description=product.description, price=product.price) for product in products]
        response = ViewAllProductsResponse(status="success",data=products_dto)
        return response

class ViewAllProductsController:
    def __init__(self, input_port: IViewAllProductsInputPort) -> None:
        self.input_port = input_port

    def handle_request(self):
        """Handles the incoming API request and delegates to the use case."""
        return self.input_port.execute()
    
view_all_products_presenter = ViewAllProductsPresenter()
view_all_products_use_case = ViewAllProducts(product_repository, view_all_products_presenter)
view_all_products_controller = ViewAllProductsController(view_all_products_use_case)

router = APIRouter()

@router.post("/view_product", response_model=ViewProductResponse)
async def view_product(request: ViewProductRequest):
    return view_product_controller.handle_request(request)

@router.get("/view_products", response_model=ViewAllProductsResponse)
async def view_products():
    return view_all_products_controller.handle_request()
