#python imports
from typing import Optional, List
import sqlite3

#dependency imports
from use_cases.product_repository import IProductRepository
from entities.product import Product

class SQLProductRepository(IProductRepository):    
    def __init__(self, db_file="framework_driver/db/shop.db"):
        """Initialize the ProductDatabase with a connection to the SQLite database."""
        self.conn = sqlite3.connect(db_file)
        self.conn.row_factory = sqlite3.Row

    def get_product_by_id(self, product_id) -> Optional[Product]:
        """Retrieve a product by its ID."""
        sql = "SELECT * FROM products WHERE id = ?"
        try:
            c = self.conn.cursor()
            c.execute(sql, (product_id,))
            r = c.fetchone()
            return Product(**r)
        except sqlite3.Error as e:
            print(e)
            return None

    def add_product(self, product):
        """Add a new product to the products table."""
        sql = "INSERT INTO products (name, price, description, quantity) VALUES (?, ?, ?, ?)"
        try:
            c = self.conn.cursor()
            c.execute(sql, product)
            self.conn.commit()
            return c.lastrowid
        except sqlite3.Error as e:
            print(e)

    def get_all_products(self) -> Optional[List[Product]]:
        """Retrieve all products."""
        sql = "SELECT * FROM products"
        try:
            c = self.conn.cursor()
            c.execute(sql)
            rows = c.fetchall()
            return [Product(**product) for product in rows]
        except sqlite3.Error as e:
            print(e)
            return []

def main():
    database = "framework_driver/db/shop.db"

    product_db = SQLProductRepository(database)

    # delete all products
    for product in product_db.get_all_products():
        product_db.delete_product(product[0])

    # name, price, quantity
    products = [("Unsichtbare Tinte", 2.5, "unsichtbar", 5),
                ("Luftdichter Sieb", 7.5, "luftdicht", 10),
                ("Geräuschloser Wecker", 15.0, "geräuschlos", 7)]

    for product in products:
        product_db.add_product(product)

    products = product_db.get_all_products()
    print("All products:")
    for product in products:
        print(product)


if __name__ == "__main__":
    main()